import java.io.*;
import java.util.Date;

import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.util.EdgeType;
import gmu.cnl.*;

public class DrawGraph {
	private String rTabName;
	// Scanner scanner = null;
	BufferedReader br;
	Graph<ASNode, ASLink> g;
	int linkCount;

	boolean init() {
		try {
			FileReader fr = new FileReader(rTabName);
			br = new BufferedReader(fr);
			// scanner = new Scanner(new File(rTabName));
			g = new SparseGraph<ASNode, ASLink>();
			linkCount = 0;
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

	}

	void read() throws IOException {
		System.out.print("Reading from file.");
		int Totalcout = 0;
		int lineCap = 10000;
		String[] lineBuf = new String[lineCap];
		boolean finished = false;
		try {
			while (!finished) {
				int i = 0;
				for (i = 0; i < lineCap; i++) {
					String entry = br.readLine();
					if (entry != null) {
						lineBuf[i] = entry.trim();
						Totalcout++;
					} else {
						finished = true;
						break;
					}
				}

				draw(lineBuf, i);

				// if(cout>=10000){
				// break;
				// }
			}
		} finally {
			System.out
					.println("The total lines of routing table: " + Totalcout);
			br.close();
		}
	}

	ASNode findASNode(int ASNo) {
		for (ASNode node : g.getVertices()) {
			if (node.getASNo() == ASNo)
				return node;
		}
		return null;
	}

	void drawSingleLine(String rTabEntry) {
		String[] list = rTabEntry.split(" ");
		rTabEntry = null;
		int length = list.length;
		try {
			ASNode previousAS = null;
			ASNode currentAS = null;
			int previousASNo = 0;
			int currentASNo = 0;
			int weight = 1;
			for (int i = length-1; i > 0; i--) {
				currentASNo = Integer.parseInt(list[i]);
				if (previousASNo == currentASNo){
					weight++;
					continue;
				}
				currentAS = findASNode(currentASNo);
				if (currentAS == null) {
					currentAS = new ASNode(currentASNo);
					g.addVertex(currentAS);
				}
				if (previousAS != null) {
					ASLink link = findASLink(previousAS, currentAS);
					if (link == null) {
						link = new ASLink(linkCount++);
						link.setWeight(weight);
						g.addEdge(link, previousAS, currentAS,
								EdgeType.UNDIRECTED);
					} else {
						int tmp = link.getWeight();
						if(weight>tmp)
							link.setWeight(weight);
						link.addReferCount();
					}
					weight = 1;
				}
				previousAS = currentAS;
				previousASNo = currentASNo;
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	void readSingleLine() throws IOException {
		System.out.print("Reading from file.");
		int cout = 0;
		try {
			String text = br.readLine();
			while (text != null) {
				drawSingleLine(text.trim());
				text = br.readLine();
				cout++;
				// if(cout>=1000000){
				// break;
				// }
			}
		} finally {
			System.out.println("The total lines of routing table: " + cout);
			br.close();
		}
	}

	ASLink findASLink(ASNode previous, ASNode current) {
		return g.findEdge(previous, current);
	}

	void draw(String[] rTabBuf, int size) {
		try {
			for (int j = 0; j < size; j++) {
				String[] list = rTabBuf[j].split(" ");
				int length = list.length;
				ASNode previousAS = null;
				ASNode currentAS = null;
				int previousASNo = 0;
				int currentASNo = 0;
				for (int i = 1; i < length; i++) {
					currentASNo = Integer.parseInt(list[i]);
					if (previousASNo == currentASNo)
						continue;
					currentAS = findASNode(currentASNo);
					if (currentAS == null) {
						currentAS = new ASNode(currentASNo);
						g.addVertex(currentAS);
					}
					if (previousAS != null) {
						ASLink link = findASLink(previousAS, currentAS);
						if (link == null) {
							link = new ASLink(linkCount++);
							g.addEdge(link, previousAS, currentAS,
									EdgeType.UNDIRECTED);
						} else
							link.addReferCount();
					}
					previousAS = currentAS;
					previousASNo = currentASNo;
				}
			}
		} catch (Exception e) {
			System.out.println("size is:" + size);
			e.printStackTrace();
		}
	}

	public DrawGraph(String rTabName) {
		// TODO Auto-generated constructor stub
		this.rTabName = rTabName;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		System.out.println((new Date()));
		String filename = args[0];// "../RouTabParse/rt.edge";
		DrawGraph drawGraph = new DrawGraph(filename);
		drawGraph.init();

		try {
			drawGraph.readSingleLine();
			FileOutputStream fos = new FileOutputStream(filename + ".graph");
			ObjectOutputStream out = new ObjectOutputStream(fos);
			out.writeObject(drawGraph.g);
			out.close();
			fos.close();
			System.out.println((new Date()));
		} catch (Exception ex) {
			ex.printStackTrace();
		}

	}

}
