import java.io.*;
import java.util.Date;

//import edu.uci.ics.jung.algorithms.importance.BetweennessCentrality;
import edu.uci.ics.jung.algorithms.scoring.BetweennessCentrality;
import edu.uci.ics.jung.algorithms.shortestpath.DijkstraShortestPath;
import edu.uci.ics.jung.graph.SparseGraph;
import org.apache.commons.collections15.Transformer;
import gmu.cnl.*;

public class ReadRTResult {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		// TODO Auto-generated method stub
		int choice = Integer.parseInt(args[0]);
		switch (choice) {
		case 1:
			if (args.length < 2) {
				System.err.println("need asfilename to parse IP deaggregation");
				System.exit(0);
			}
			calcIPdeagreggationRate(args[1]);
			break;
		case 2:
			if (args.length < 2) {
				System.err.println("need asfilename to get edge as number");
				System.exit(0);
			}
			getASNOandPosition(args[1]);
			break;
		case 3:
			if (args.length < 3) {
				System.err
						.println("need asnofile and graphfilename to find the shortest path");
				System.exit(0);
			}
			findEdgePairShortestPath(args[1], args[2]);
			break;
		case 4:
			if (args.length < 2) {
				System.err
						.println("need asfilename to find the edge and transit ASes");
				System.exit(0);
			}
			findEdgeTransitAS(args[1]);
			break;
		case 5:
			if (args.length < 3) {
				System.err.println("need two file names to compare");
				System.exit(0);
			}
			findDiff(args[1], args[2]);
			break;
		case 6:
			if (args.length < 2) {
				System.err.println("need a file name to debug");
				System.exit(0);
			}
			checkIP(args[1]);
			break;
		case 7:
			if (args.length < 2) {
				System.err.println("need graph to calcuate betweeness and degree");
				System.exit(0);
			}
			calcbetweenessdegree(args[1]);
			break;
		case 8:
			if (args.length < 2) {
				System.err.println("need graph to calcuate diameter");
				System.exit(0);
			}
			calcDiameter(args[1]);
			break;
		case 9:
			if (args.length < 3) {
				System.err.println("need graph and edge as to get sub graph");
				System.exit(0);
			}
			getSubGraph(args[1], args[2]);
			break;
		case 10:
			if (args.length < 2) {
				System.err.println("need graph to get all the links referCount");
				System.exit(0);
			}
			getGraphLinkReferCount(args[1]);
			break;
		}
	}
	static int getTotalLineNo(BufferedReader br) throws Exception {
		int totalLine = 0;
		while(br.readLine()!=null) {
			totalLine++;
		}
		return totalLine;
	}
	static void getGraphLinkReferCount(String graphname) throws Exception {
		FileInputStream fis = new FileInputStream(graphname);
		ObjectInputStream in = new ObjectInputStream(fis);
		SparseGraph<ASNode, ASLink> g = (SparseGraph<ASNode, ASLink>) in.readObject();
		//System.out.println(g.getVertexCount());
		in.close();
		fis.close();
		System.out.println(new Date() + "begin to get links information");
		FileOutputStream fos = new FileOutputStream(graphname + ".link");
		PrintStream ps = new PrintStream(fos);
		Object [] nodes = g.getVertices().toArray();
		int len = nodes.length;
		for (int i = 0; i < len-1; i++) {
			ASNode node1 = (ASNode)nodes[i];
			for (int j = i + 1; j < len; j++) {
				ASNode node2 = (ASNode)nodes[j];
				ASLink link = g.findEdge(node1, node2);
				if(link!=null)
					ps.println(node1.getASNo()+" "+node2.getASNo()+
							" "+link.getReferCount()+" "+link.getWeight());
			}
		}
		System.out.println(new Date() + "job accomplished");
	}
	static void getSubGraph(String asnofile, String graphname) throws Exception {
		FileReader fr = new FileReader(asnofile);
		BufferedReader br = new BufferedReader(fr);
		int len = getTotalLineNo(br);
		fr.close();
		int[] edgeas = new int[len];
		fr = new FileReader(asnofile);
		br = new BufferedReader(fr);
		for (int i = 0; i < len; i++) {
			String line = br.readLine().trim();
			edgeas[i] = Integer.parseInt(line);
		}
		br.close();
		fr.close();
		
		FileInputStream fis = new FileInputStream(graphname);
		ObjectInputStream in = new ObjectInputStream(fis);
		SparseGraph<ASNode, ASLink> g = (SparseGraph<ASNode, ASLink>) in.readObject();
		//System.out.println(g.getVertexCount());
		in.close();
		fis.close();
		System.out.println(new Date() + "begin to get sub graph");
		Object [] nodes = g.getVertices().toArray();
		int len1 = nodes.length;
		for (int i = 0; i < len1; i++) {
			ASNode node = (ASNode)nodes[i];
			for(int j=0;j<len;j++){
				if(node.getASNo()==edgeas[j])
				{
					g.removeVertex(node);
					break;
				}
			}
		}
//		for(ASNode node : g.getVertices()){
//			for(int i=0;i<len;i++){
//				if(node.getASNo()==edgeas[i])
//				{
//					g.removeVertex(node);
//					break;
//				}
//			}
//		}
		
		FileOutputStream fos = new FileOutputStream(graphname + ".sub");
		ObjectOutputStream out = new ObjectOutputStream(fos);
		out.writeObject(g);
		out.close();
		fos.close();
		System.out.println(new Date() + "job accomplished");
	}
	static void calcDiameter(String graphname) throws Exception {
		System.out.println(new Date() + " begin to read graph file");
		// read the graph file
		FileInputStream fis = new FileInputStream(graphname);
		ObjectInputStream in = new ObjectInputStream(fis);
		SparseGraph<ASNode, ASLink> g = (SparseGraph<ASNode, ASLink>) in.readObject();
		in.close();
		fis.close();
		// use to find weighted shortest path.
		Transformer<ASLink, Double> wtTransformer = new Transformer<ASLink, Double>() {
			public Double transform(ASLink link) {
				return 1.0 * link.getWeight();
			}
		};
		DijkstraShortestPath<ASNode, ASLink> alg1 = new DijkstraShortestPath(g);
		DijkstraShortestPath<ASNode, ASLink> alg2 = new DijkstraShortestPath(g, wtTransformer);
		// alg1.enableCaching(false);
		// alg2.enableCaching(false);
		System.out.println(new Date() + " begin to find diameter");
		int diameter = 0;
		int wdiameter = 0;
		Object [] nodes = g.getVertices().toArray();
		int len = nodes.length;
		for (int i = 0; i < len-1; i++) {
			ASNode node1 = (ASNode)nodes[i];
			for (int j = i + 1; j < len; j++) {
				ASNode node2 = (ASNode)nodes[j];
				int l1 = alg1.getDistance(node1, node2).intValue();
				int l2 = alg2.getDistance(node1, node2).intValue();
				if(l1>diameter)
					diameter = l1;
				if(l2>wdiameter)
					wdiameter = l2;
			}
			alg1.reset();
			alg2.reset();
		}
		System.out.println(diameter + " " +wdiameter);
		System.out.println(new Date() + " task accomplished");
	}
	static void calcbetweenessdegree(String graphname) throws Exception {
		System.out.println("Task starts "+new Date());
		FileInputStream fis = new FileInputStream(graphname);
		ObjectInputStream in = new ObjectInputStream(fis);
		SparseGraph<ASNode, ASLink> g = (SparseGraph<ASNode, ASLink>) in.readObject();
		in.close();
		fis.close();
		FileOutputStream fos = new FileOutputStream(graphname + ".bd");
		PrintStream ps = new PrintStream(fos);
		BetweennessCentrality ranker = new BetweennessCentrality(g);
		//ranker.setNormalizeRankings(true);
		//ranker.evaluate();
		//ranker.printRankings(true, false);
		for (ASNode node : g.getVertices()) {
			double between=ranker.getVertexScore(node);
			int degree = g.degree(node);
			ps.println(node.getASNo()+" "+degree+" "+between);
		}
		System.out.println("Task ends "+new Date());
		
	}
	static void findDiff(String total, String edge) throws Exception {
		FileOutputStream fos = new FileOutputStream(total + ".tran");
		PrintStream ps = new PrintStream(fos);
		FileReader fr1 = new FileReader(total);
		BufferedReader br1 = new BufferedReader(fr1);
		FileReader fr2 = new FileReader(edge);
		BufferedReader br2 = new BufferedReader(fr2);
		String line1 = br1.readLine();
		String line2 = br2.readLine();
		while (line1 != null) {
			while (!line1.equals(line2)) {
				ps.println(line1);
				line1 = br1.readLine();
			}
			line1 = br1.readLine();
			line2 = br2.readLine();
		}
	}

	static void checkIP(String asfilename) throws Exception {
		LinkSimpleAS linkAS = new LinkSimpleAS();
		FileInputStream fis = new FileInputStream(asfilename);
		ObjectInputStream in = new ObjectInputStream(fis);
		linkAS.readExternal(in);
		in.close();
		fis.close();
		SimpleAS current = linkAS.first;
		FileOutputStream fos = new FileOutputStream(asfilename + ".ip");
		PrintStream ps = new PrintStream(fos);
		while (current != null) {
			if (current.addrList.size() > 100) {
				ps.println(current.addrList);
			}
			current = current.next;
		}
	}

	static void findEdgeTransitAS(String asfilename) throws Exception {
		LinkSimpleAS linkAS = new LinkSimpleAS();
		FileInputStream fis = new FileInputStream(asfilename);
		ObjectInputStream in = new ObjectInputStream(fis);
		linkAS.readExternal(in);
		in.close();
		fis.close();
		LinkSimpleAS edgeAS = new LinkSimpleAS();
		SimpleAS current = linkAS.first;
		SimpleAS previous = null;
		while (current != null) {
			boolean isEdge = true;
			boolean isPureTran = true;
			int posLength = current.positionList.size();
			int i = 0;
			while ((isEdge | isPureTran) & i < posLength) {
				boolean flag = (current.positionList.get(i) <= 3);
				isEdge = isEdge & flag;
				isPureTran = isPureTran & (!flag);
				i++;
			}
			if (isEdge) {
				linkAS.delete(previous, current);
				SimpleAS tmp = current.next;
				edgeAS.insert(current);
				current = tmp;
				tmp = null;

			}
			// else if (isPureTran) {
			// linkAS.delete(previous, current);
			// SimpleAS tmp = current.next;
			// pureTranAS.insert(current);
			// current = tmp;
			// tmp = null;
			// }
			else {
				previous = current;
				current = current.next;
			}
		}
		System.out.println("The No. of Edge is:" + edgeAS.length);
		System.out.println("The No. of Transit is:" + linkAS.length);
		// System.out.println("The No. of Left AS is:" + rtParse.head.length);
		System.out.println((new Date()));
		// Write to disk with FileOutputStream
		FileOutputStream f_out = new FileOutputStream(asfilename + ".edge");
		ObjectOutputStream obj_out = new ObjectOutputStream(f_out);
		edgeAS.writeExternal(obj_out);
		obj_out.flush();
		obj_out.close();
		f_out.close();

		f_out = new FileOutputStream(asfilename + ".tran");
		obj_out = new ObjectOutputStream(f_out);
		linkAS.writeExternal(obj_out);
		obj_out.flush();
		obj_out.close();
		f_out.close();
	}
// use to get ASes No and the maximum position number as well as 
// the length of position and if use AS path prepending
	static void getASNOandPosition(String asfilename) throws Exception {
		FileOutputStream fos = new FileOutputStream(asfilename + ".asno");
		PrintStream ps = new PrintStream(fos);
		System.out.println(new Date() + " begin to read asfile");
		LinkSimpleAS linkAS = null;
		FileInputStream fis = new FileInputStream(asfilename);
		ObjectInputStream in = new ObjectInputStream(fis);
		linkAS = new LinkSimpleAS();
		linkAS.readExternal(in);
		in.close();
		fis.close();
		System.out.println(new Date() + " begin to get AS No. as well as AS position");
		SimpleAS current = linkAS.first;
		while (current != null) {
			if(current.isASPathPrepended)
				ps.println(current.ASNO+ " "+current.level+" "+current.positionList.size()+" 1");
			else
				ps.println(current.ASNO+ " "+current.level+" "+current.positionList.size()+" 0");
			current=current.next;
		}
		ps.close();
		fos.close();
	}

	static void findEdgePairShortestPath(String asnofile, String graphname)
			throws Exception {
		FileOutputStream fos = new FileOutputStream(graphname + ".spath");
		PrintStream ps = new PrintStream(fos);
		FileReader fr = new FileReader(asnofile);
		BufferedReader br = new BufferedReader(fr);
		String line = br.readLine().trim();

		int len = Integer.parseInt(line);
		int[] edgeas = new int[len];
		for (int i = 0; i < len; i++) {
			line = br.readLine().trim();
			edgeas[i] = Integer.parseInt(line);
		}
		br.close();
		fr.close();

		System.out.println(new Date() + " begin to read graph file");
		// read the graph file
		FileInputStream fis = new FileInputStream(graphname);
		ObjectInputStream in = new ObjectInputStream(fis);
		SparseGraph<ASNode, ASLink> g = (SparseGraph<ASNode, ASLink>) in.readObject();
		in.close();
		fis.close();
		// use to find weighted shortest path.
		Transformer<ASLink, Double> wtTransformer = new Transformer<ASLink, Double>() {
			public Double transform(ASLink link) {
				return 1.0 * link.getWeight();
			}
		};
		DijkstraShortestPath<ASNode, ASLink> alg1 = new DijkstraShortestPath(g);
		DijkstraShortestPath<ASNode, ASLink> alg2 = new DijkstraShortestPath(g,
				wtTransformer);
		// alg1.enableCaching(false);
		// alg2.enableCaching(false);
		System.out.println(new Date() + " begin to find shortest path");
		for (int i = 0; i < len - 1; i++) {
			ASNode node1 = findASNode(edgeas[i], g);
			for (int j = i + 1; j < len; j++) {
				ASNode node2 = findASNode(edgeas[j], g);
				Number l1 = alg1.getDistance(node1, node2);
				Number l2 = alg2.getDistance(node1, node2);
				ps.println(l1 + " " + l2);
			}
			alg1.reset();
			alg2.reset();
		}
		System.out.println(new Date() + " task accomplished");
	}

	static ASNode findASNode(int ASNo, SparseGraph<ASNode, ASLink> g) {
		for (ASNode node : g.getVertices()) {
			if (node.getASNo() == ASNo)
				return node;
		}
		return null;
	}

	static void calcIPdeagreggationRate(String asfilename) throws Exception {
		System.out.println(new Date());
		FileOutputStream fos = new FileOutputStream(asfilename + ".derate");
		PrintStream ps = new PrintStream(fos);

		LinkSimpleAS linkAS = null;
		FileInputStream fis = new FileInputStream(asfilename);
		ObjectInputStream in = new ObjectInputStream(fis);
		linkAS = new LinkSimpleAS();
		linkAS.readExternal(in);
		in.close();
		fis.close();

		int len = linkAS.length;
		MyIPAddrMerge ipMerge = null;
		SimpleAS current = linkAS.first;
		for (int i = 0; i < len; i++) {
			if (!current.addrList.isEmpty()) {
				ipMerge = new MyIPAddrMerge(current.addrList);
				ipMerge.doOverAllIPMerge();
				ps.println(current.ASNO + " " + current.level + " "
						+ ipMerge.addrLengthBeforeMerge + " "
						+ ipMerge.addrLengthAfterMerge + " "
						+ ipMerge.deaggregationRate());
			}
			current = current.next;
		}
		System.out.println(new Date());
		// LinkedList<String> addrList = new LinkedList<String>();
		// addrList.add("125.0.1.0/25");
		// addrList.add("125.0.1.0/26");
		// addrList.add("125.0.1.128/25");
		// addrList.add("125.0.0.0/24");
		// System.out.println(addrList);
		// MyIPAddrMerge test = new MyIPAddrMerge(addrList);
		// test.doOverAllIPMerge();
		// System.out.println(test.addrList);
	}

}
